@@ -1,12 +1,11 @@
-"""
-This package contains interfaces and functionality to compute pair-wise document
-similarities within a corpus of documents.
+"""This package contains interfaces and functionality to compute pair-wise document similarities within a corpus
+of documents.
 """
 
-from gensim import parsing, matutils, interfaces, corpora, models, similarities, summarization, utils  # noqa:F401
+from gensim import parsing, corpora, matutils, interfaces, models, similarities, summarization, utils  # noqa:F401
 import logging
 
-__version__ = '3.0.0'
+__version__ = '3.4.0'
 
 
 class NullHandler(logging.Handler):
