@@ -14,9 +14,6 @@ needed are thus quite similar.
 
 import logging
 import unittest
-import os
-import os.path
-import tempfile
 import numbers
 from os import remove
 
@@ -27,6 +24,8 @@ from gensim.corpora import mmcorpus, Dictionary
 from gensim.models import atmodel
 from gensim import matutils
 from gensim.test import basetmtests
+from gensim.test.utils import (datapath,
+    get_tmpfile, common_texts, common_dictionary as dictionary, common_corpus as corpus)
 
 # TODO:
 # Test that computing the bound on new unseen documents works as expected (this is somewhat different
@@ -36,23 +35,6 @@ from gensim.test import basetmtests
 # increases the bound.
 # Test that models are compatiple across versions, as done in LdaModel.
 
-module_path = os.path.dirname(__file__)  # needed because sample data files are located in the same folder
-datapath = lambda fname: os.path.join(module_path, 'test_data', fname)
-
-# set up vars used in testing ("Deerwester" from the web tutorial)
-texts = [
-    ['human', 'interface', 'computer'],
-    ['survey', 'user', 'computer', 'system', 'response', 'time'],
-    ['eps', 'user', 'interface', 'system'],
-    ['system', 'human', 'system', 'eps'],
-    ['user', 'response', 'time'],
-    ['trees'],
-    ['graph', 'trees'],
-    ['graph', 'minors', 'trees'],
-    ['graph', 'minors', 'survey']
-]
-dictionary = Dictionary(texts)
-corpus = [dictionary.doc2bow(text) for text in texts]
 
 # Assign some authors randomly to the documents above.
 author2doc = {
@@ -61,6 +43,7 @@ author2doc = {
     'jack': [0, 2, 4, 6, 8],
     'jill': [1, 3, 5, 7]
 }
+
 doc2author = {
     0: ['john', 'jack'],
     1: ['john', 'jill'],
@@ -76,18 +59,12 @@ doc2author = {
 # More data with new and old authors (to test update method).
 # Although the text is just a subset of the previous, the model
 # just sees it as completely new data.
-texts_new = texts[0:3]
+texts_new = common_texts[0:3]
 author2doc_new = {'jill': [0], 'bob': [0, 1], 'sally': [1, 2]}
 dictionary_new = Dictionary(texts_new)
 corpus_new = [dictionary_new.doc2bow(text) for text in texts_new]
 
 
-def testfile(test_fname=''):
-    # temporary data will be stored to this file
-    fname = 'gensim_models_' + test_fname + '.tst'
-    return os.path.join(tempfile.gettempdir(), fname)
-
-
 class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
     def setUp(self):
         self.corpus = mmcorpus.MmCorpus(datapath('testcorpus.mm'))
@@ -113,7 +90,8 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
             # output of the model slightly.
             vec = matutils.sparse2full(jill_topics, 2)  # convert to dense vector, for easier equality tests
             expected = [0.91, 0.08]
-            passed = np.allclose(sorted(vec), sorted(expected), atol=1e-1)  # must contain the same values, up to re-ordering
+            # must contain the same values, up to re-ordering
+            passed = np.allclose(sorted(vec), sorted(expected), atol=1e-1)
             if passed:
                 break
             logging.warning(
@@ -263,7 +241,8 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
             # output of the model slightly.
             vec = matutils.sparse2full(jill_topics, 2)  # convert to dense vector, for easier equality tests
             expected = [0.91, 0.08]
-            passed = np.allclose(sorted(vec), sorted(expected), atol=1e-1)  # must contain the same values, up to re-ordering
+            # must contain the same values, up to re-ordering
+            passed = np.allclose(sorted(vec), sorted(expected), atol=1e-1)
 
             # Delete the MmCorpus used for serialization inside the author-topic model.
             remove(datapath('testcorpus_serialization.mm'))
@@ -475,30 +454,32 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         # long message includes the original error message with a custom one
         self.longMessage = True
         # construct what we expect when passes aren't involved
-        test_rhots = list()
+        test_rhots = []
         model = self.class_(id2word=dictionary, chunksize=1, num_topics=2)
-        final_rhot = lambda: pow(model.offset + (1 * model.num_updates) / model.chunksize, -model.decay)
+
+        def final_rhot(model):
+            return pow(model.offset + (1 * model.num_updates) / model.chunksize, -model.decay)
 
         # generate 5 updates to test rhot on
-        for x in range(5):
+        for _ in range(5):
             model.update(corpus, author2doc)
-            test_rhots.append(final_rhot())
+            test_rhots.append(final_rhot(model))
 
         for passes in [1, 5, 10, 50, 100]:
             model = self.class_(id2word=dictionary, chunksize=1, num_topics=2, passes=passes)
-            self.assertEqual(final_rhot(), 1.0)
+            self.assertEqual(final_rhot(model), 1.0)
             # make sure the rhot matches the test after each update
             for test_rhot in test_rhots:
                 model.update(corpus, author2doc)
 
                 msg = "{}, {}, {}".format(passes, model.num_updates, model.state.numdocs)
-                self.assertAlmostEqual(final_rhot(), test_rhot, msg=msg)
+                self.assertAlmostEqual(final_rhot(model), test_rhot, msg=msg)
 
             self.assertEqual(model.state.numdocs, len(corpus) * len(test_rhots))
             self.assertEqual(model.num_updates, len(corpus) * len(test_rhots))
 
     def testPersistence(self):
-        fname = testfile()
+        fname = get_tmpfile('gensim_models_atmodel.tst')
         model = self.model
         model.save(fname)
         model2 = self.class_.load(fname)
@@ -507,7 +488,7 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         self.assertTrue(np.allclose(model.state.gamma, model2.state.gamma))
 
     def testPersistenceIgnore(self):
-        fname = testfile('testPersistenceIgnore')
+        fname = get_tmpfile('gensim_models_atmodel_testPersistenceIgnore.tst')
         model = atmodel.AuthorTopicModel(corpus, author2doc=author2doc, num_topics=2)
         model.save(fname, ignore='id2word')
         model2 = atmodel.AuthorTopicModel.load(fname)
@@ -518,7 +499,7 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         self.assertTrue(model2.id2word is None)
 
     def testPersistenceCompressed(self):
-        fname = testfile() + '.gz'
+        fname = get_tmpfile('gensim_models_atmodel.tst.gz')
         model = self.model
         model.save(fname)
         model2 = self.class_.load(fname, mmap=None)
@@ -533,7 +514,7 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         self.assertTrue(np.allclose(jill_topics, jill_topics2))
 
     def testLargeMmap(self):
-        fname = testfile()
+        fname = get_tmpfile('gensim_models_atmodel.tst')
         model = self.model
 
         # simulate storing large arrays separately
@@ -553,7 +534,7 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         self.assertTrue(np.allclose(jill_topics, jill_topics2))
 
     def testLargeMmapCompressed(self):
-        fname = testfile() + '.gz'
+        fname = get_tmpfile('gensim_models_atmodel.tst.gz')
         model = self.model
 
         # simulate storing large arrays separately
@@ -562,6 +543,20 @@ class TestAuthorTopicModel(unittest.TestCase, basetmtests.TestBaseTopicModel):
         # test loading the large model arrays with mmap
         self.assertRaises(IOError, self.class_.load, fname, mmap='r')
 
+    def testDtypeBackwardCompatibility(self):
+        atmodel_3_0_1_fname = datapath('atmodel_3_0_1_model')
+        expected_topics = [(0, 0.068200842977296727), (1, 0.93179915702270333)]
+
+        # save model to use in test
+        # self.model.save(atmodel_3_0_1_fname)
+
+        # load a model saved using a 3.0.1 version of Gensim
+        model = self.class_.load(atmodel_3_0_1_fname)
+
+        # and test it on a predefined document
+        topics = model['jane']
+        self.assertTrue(np.allclose(expected_topics, topics))
+
 
 if __name__ == '__main__':
     logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.DEBUG)
