@@ -20,21 +20,10 @@ import unittest
 import numpy as np
 
 from gensim.corpora import (bleicorpus, mmcorpus, lowcorpus, svmlightcorpus,
-                            ucicorpus, malletcorpus, textcorpus, indexedcorpus)
+                            ucicorpus, malletcorpus, textcorpus, indexedcorpus, wikicorpus)
 from gensim.interfaces import TransformedCorpus
 from gensim.utils import to_unicode
-
-# needed because sample data files are located in the same folder
-module_path = os.path.dirname(__file__)
-
-
-def datapath(fname):
-    return os.path.join(module_path, 'test_data', fname)
-
-
-def testfile():
-    # temporary data will be stored to this file
-    return os.path.join(tempfile.gettempdir(), 'gensim_corpus.tst')
+from gensim.test.utils import datapath, get_tmpfile
 
 
 class DummyTransformer(object):
@@ -61,7 +50,7 @@ class CorpusTestCase(unittest.TestCase):
 
     def tearDown(self):
         # remove all temporary test files
-        fname = testfile()
+        fname = get_tmpfile('gensim_corpus.tst')
         extensions = ['', '', '.bz2', '.gz', '.index', '.vocab']
         for ext in itertools.permutations(extensions, 2):
             try:
@@ -93,13 +82,14 @@ class CorpusTestCase(unittest.TestCase):
         self.assertEqual(len(corpus), 9)
 
     def test_empty_input(self):
-        with open(testfile(), 'w') as f:
+        tmpf = get_tmpfile('gensim_corpus.tst')
+        with open(tmpf, 'w') as f:
             f.write('')
 
-        with open(testfile() + '.vocab', 'w') as f:
+        with open(tmpf + '.vocab', 'w') as f:
             f.write('')
 
-        corpus = self.corpus_class(testfile())
+        corpus = self.corpus_class(tmpf)
         self.assertEqual(len(corpus), 0)
 
         docs = list(corpus)
@@ -107,22 +97,24 @@ class CorpusTestCase(unittest.TestCase):
 
     def test_save(self):
         corpus = self.TEST_CORPUS
+        tmpf = get_tmpfile('gensim_corpus.tst')
 
         # make sure the corpus can be saved
-        self.corpus_class.save_corpus(testfile(), corpus)
+        self.corpus_class.save_corpus(tmpf, corpus)
 
         # and loaded back, resulting in exactly the same corpus
-        corpus2 = list(self.corpus_class(testfile()))
+        corpus2 = list(self.corpus_class(tmpf))
         self.assertEqual(corpus, corpus2)
 
     def test_serialize(self):
         corpus = self.TEST_CORPUS
+        tmpf = get_tmpfile('gensim_corpus.tst')
 
         # make sure the corpus can be saved
-        self.corpus_class.serialize(testfile(), corpus)
+        self.corpus_class.serialize(tmpf, corpus)
 
         # and loaded back, resulting in exactly the same corpus
-        corpus2 = self.corpus_class(testfile())
+        corpus2 = self.corpus_class(tmpf)
         self.assertEqual(corpus, list(corpus2))
 
         # make sure the indexing corpus[i] works
@@ -133,13 +125,14 @@ class CorpusTestCase(unittest.TestCase):
         # after deserialisation
         if isinstance(corpus, indexedcorpus.IndexedCorpus):
             idx = [1, 3, 5, 7]
-            self.assertEquals(corpus[idx], corpus2[idx])
+            self.assertEqual(corpus[idx], corpus2[idx])
 
     def test_serialize_compressed(self):
         corpus = self.TEST_CORPUS
+        tmpf = get_tmpfile('gensim_corpus.tst')
 
         for extension in ['.gz', '.bz2']:
-            fname = testfile() + extension
+            fname = tmpf + extension
             # make sure the corpus can be saved
             self.corpus_class.serialize(fname, corpus)
 
@@ -202,10 +195,10 @@ class CorpusTestCase(unittest.TestCase):
 
         # check sliced corpora that use fancy indexing
         c = corpus[[1, 3, 4]]
-        self.assertEquals([d for i, d in enumerate(docs) if i in [1, 3, 4]], list(c))
-        self.assertEquals([d for i, d in enumerate(docs) if i in [1, 3, 4]], list(c))
-        self.assertEquals(len(corpus[[0, 1, -1]]), 3)
-        self.assertEquals(len(corpus[np.asarray([0, 1, -1])]), 3)
+        self.assertEqual([d for i, d in enumerate(docs) if i in [1, 3, 4]], list(c))
+        self.assertEqual([d for i, d in enumerate(docs) if i in [1, 3, 4]], list(c))
+        self.assertEqual(len(corpus[[0, 1, -1]]), 3)
+        self.assertEqual(len(corpus[np.asarray([0, 1, -1])]), 3)
 
         # check that TransformedCorpus supports indexing when the underlying
         # corpus does, and throws an error otherwise
@@ -214,7 +207,7 @@ class CorpusTestCase(unittest.TestCase):
             self.assertEqual(corpus_[0][0][1], docs[0][0][1] + 1)
             self.assertRaises(ValueError, _get_slice, corpus_, {1})
             transformed_docs = [val + 1 for i, d in enumerate(docs) for _, val in d if i in [1, 3, 4]]
-            self.assertEquals(transformed_docs, list(v for doc in corpus_[[1, 3, 4]] for _, v in doc))
+            self.assertEqual(transformed_docs, list(v for doc in corpus_[[1, 3, 4]] for _, v in doc))
             self.assertEqual(3, len(corpus_[[1, 3, 4]]))
         else:
             self.assertRaises(RuntimeError, _get_slice, corpus_, [1, 3, 4])
@@ -222,10 +215,44 @@ class CorpusTestCase(unittest.TestCase):
             self.assertRaises(RuntimeError, _get_slice, corpus_, 1.0)
 
 
-class TestMmCorpus(CorpusTestCase):
+class TestMmCorpusWithIndex(CorpusTestCase):
     def setUp(self):
         self.corpus_class = mmcorpus.MmCorpus
-        self.corpus = self.corpus_class(datapath('testcorpus.mm'))
+        self.corpus = self.corpus_class(datapath('test_mmcorpus_with_index.mm'))
+        self.file_extension = '.mm'
+
+    def test_serialize_compressed(self):
+        # MmCorpus needs file write with seek => doesn't support compressed output (only input)
+        pass
+
+    def test_closed_file_object(self):
+        file_obj = open(datapath('testcorpus.mm'))
+        f = file_obj.closed
+        mmcorpus.MmCorpus(file_obj)
+        s = file_obj.closed
+        self.assertEqual(f, 0)
+        self.assertEqual(s, 0)
+
+    def test_load(self):
+        self.assertEqual(self.corpus.num_docs, 9)
+        self.assertEqual(self.corpus.num_terms, 12)
+        self.assertEqual(self.corpus.num_nnz, 28)
+
+        # confirm we can iterate and that document values match expected for first three docs
+        it = iter(self.corpus)
+        self.assertEqual(next(it), [(0, 1.0), (1, 1.0), (2, 1.0)])
+        self.assertEqual(next(it), [(0, 1.0), (3, 1.0), (4, 1.0), (5, 1.0), (6, 1.0), (7, 1.0)])
+        self.assertEqual(next(it), [(2, 1.0), (5, 1.0), (7, 1.0), (8, 1.0)])
+
+        # confirm that accessing document by index works
+        self.assertEqual(self.corpus[3], [(1, 1.0), (5, 2.0), (8, 1.0)])
+        self.assertEqual(tuple(self.corpus.index), (97, 121, 169, 201, 225, 249, 258, 276, 303))
+
+
+class TestMmCorpusNoIndex(CorpusTestCase):
+    def setUp(self):
+        self.corpus_class = mmcorpus.MmCorpus
+        self.corpus = self.corpus_class(datapath('test_mmcorpus_no_index.mm'))
         self.file_extension = '.mm'
 
     def test_serialize_compressed(self):
@@ -236,7 +263,79 @@ class TestMmCorpus(CorpusTestCase):
         self.assertEqual(self.corpus.num_docs, 9)
         self.assertEqual(self.corpus.num_terms, 12)
         self.assertEqual(self.corpus.num_nnz, 28)
-        self.assertEqual(tuple(self.corpus.index), (97, 121, 169, 201, 225, 249, 258, 276, 303))
+
+        # confirm we can iterate and that document values match expected for first three docs
+        it = iter(self.corpus)
+        self.assertEqual(next(it), [(0, 1.0), (1, 1.0), (2, 1.0)])
+        self.assertEqual(next(it), [])
+        self.assertEqual(next(it), [(2, 0.42371910849), (5, 0.6625174), (7, 1.0), (8, 1.0)])
+
+        # confirm that accessing document by index fails
+        self.assertRaises(RuntimeError, lambda: self.corpus[3])
+
+
+class TestMmCorpusNoIndexGzip(CorpusTestCase):
+    def setUp(self):
+        self.corpus_class = mmcorpus.MmCorpus
+        self.corpus = self.corpus_class(datapath('test_mmcorpus_no_index.mm.gz'))
+        self.file_extension = '.mm'
+
+    def test_serialize_compressed(self):
+        # MmCorpus needs file write with seek => doesn't support compressed output (only input)
+        pass
+
+    def test_load(self):
+        self.assertEqual(self.corpus.num_docs, 9)
+        self.assertEqual(self.corpus.num_terms, 12)
+        self.assertEqual(self.corpus.num_nnz, 28)
+
+        # confirm we can iterate and that document values match expected for first three docs
+        it = iter(self.corpus)
+        self.assertEqual(next(it), [(0, 1.0), (1, 1.0), (2, 1.0)])
+        self.assertEqual(next(it), [])
+        self.assertEqual(next(it), [(2, 0.42371910849), (5, 0.6625174), (7, 1.0), (8, 1.0)])
+
+        # confirm that accessing document by index fails
+        self.assertRaises(RuntimeError, lambda: self.corpus[3])
+
+
+class TestMmCorpusNoIndexBzip(CorpusTestCase):
+    def setUp(self):
+        self.corpus_class = mmcorpus.MmCorpus
+        self.corpus = self.corpus_class(datapath('test_mmcorpus_no_index.mm.bz2'))
+        self.file_extension = '.mm'
+
+    def test_serialize_compressed(self):
+        # MmCorpus needs file write with seek => doesn't support compressed output (only input)
+        pass
+
+    def test_load(self):
+        self.assertEqual(self.corpus.num_docs, 9)
+        self.assertEqual(self.corpus.num_terms, 12)
+        self.assertEqual(self.corpus.num_nnz, 28)
+
+        # confirm we can iterate and that document values match expected for first three docs
+        it = iter(self.corpus)
+        self.assertEqual(next(it), [(0, 1.0), (1, 1.0), (2, 1.0)])
+        self.assertEqual(next(it), [])
+        self.assertEqual(next(it), [(2, 0.42371910849), (5, 0.6625174), (7, 1.0), (8, 1.0)])
+
+        # confirm that accessing document by index fails
+        self.assertRaises(RuntimeError, lambda: self.corpus[3])
+
+
+class TestMmCorpusCorrupt(CorpusTestCase):
+    def setUp(self):
+        self.corpus_class = mmcorpus.MmCorpus
+        self.corpus = self.corpus_class(datapath('test_mmcorpus_corrupt.mm'))
+        self.file_extension = '.mm'
+
+    def test_serialize_compressed(self):
+        # MmCorpus needs file write with seek => doesn't support compressed output (only input)
+        pass
+
+    def test_load(self):
+        self.assertRaises(ValueError, lambda: [doc for doc in self.corpus])
 
 
 class TestSvmLightCorpus(CorpusTestCase):
@@ -252,7 +351,7 @@ class TestBleiCorpus(CorpusTestCase):
 
     def test_save_format_for_dtm(self):
         corpus = [[(1, 1.0)], [], [(0, 5.0), (2, 1.0)], []]
-        test_file = testfile()
+        test_file = get_tmpfile('gensim_corpus.tst')
         self.corpus_class.save_corpus(test_file, corpus)
         with open(test_file) as f:
             for line in f:
@@ -407,6 +506,171 @@ class TestTextCorpus(CorpusTestCase):
         pass
 
 
+# Needed for the test_custom_tokenizer is the TestWikiCorpus class.
+# Cannot be nested due to serializing.
+def custom_tokenizer(content, token_min_len=2, token_max_len=15, lower=True):
+    return [
+        to_unicode(token.lower()) if lower else to_unicode(token) for token in content.split()
+        if token_min_len <= len(token) <= token_max_len and not token.startswith('_')
+    ]
+
+
+class TestWikiCorpus(TestTextCorpus):
+    def setUp(self):
+        self.corpus_class = wikicorpus.WikiCorpus
+        self.file_extension = '.xml.bz2'
+        self.fname = datapath('testcorpus.' + self.file_extension.lstrip('.'))
+        self.enwiki = datapath('enwiki-latest-pages-articles1.xml-p000000010p000030302-shortened.bz2')
+
+    def test_default_preprocessing(self):
+        expected = ['computer', 'human', 'interface']
+        corpus = self.corpus_class(self.fname, article_min_tokens=0)
+        first_text = next(corpus.get_texts())
+        self.assertEqual(expected, first_text)
+
+    def test_len(self):
+        # When there is no min_token limit all 9 articles must be registered.
+        corpus = self.corpus_class(self.fname, article_min_tokens=0)
+        all_articles = corpus.get_texts()
+        assert (len(list(all_articles)) == 9)
+
+        # With a huge min_token limit, all articles should be filtered out.
+        corpus = self.corpus_class(self.fname, article_min_tokens=100000)
+        all_articles = corpus.get_texts()
+        assert (len(list(all_articles)) == 0)
+
+    def test_load_with_metadata(self):
+        corpus = self.corpus_class(self.fname, article_min_tokens=0)
+        corpus.metadata = True
+        self.assertEqual(len(corpus), 9)
+
+        docs = list(corpus)
+        self.assertEqual(len(docs), 9)
+
+        for i, docmeta in enumerate(docs):
+            doc, metadata = docmeta
+            article_no = i + 1  # Counting IDs from 1
+            self.assertEqual(metadata[0], str(article_no))
+            self.assertEqual(metadata[1], 'Article%d' % article_no)
+
+    def test_load(self):
+        corpus = self.corpus_class(self.fname, article_min_tokens=0)
+
+        docs = list(corpus)
+        # the deerwester corpus always has nine documents
+        self.assertEqual(len(docs), 9)
+
+    def test_first_element(self):
+        """
+        First two articles in this sample are
+        1) anarchism
+        2) autism
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1)
+
+        texts = corpus.get_texts()
+        self.assertTrue(u'anarchism' in next(texts))
+        self.assertTrue(u'autism' in next(texts))
+
+    def test_unicode_element(self):
+        """
+        First unicode article in this sample is
+        1) папа
+        """
+        bgwiki = datapath('bgwiki-latest-pages-articles-shortened.xml.bz2')
+        corpus = self.corpus_class(bgwiki)
+        texts = corpus.get_texts()
+        self.assertTrue(u'папа' in next(texts))
+
+    def test_custom_tokenizer(self):
+        """
+        define a custom tokenizer function and use it
+        """
+        wc = self.corpus_class(self.enwiki, processes=1, lemmatize=False, tokenizer_func=custom_tokenizer,
+                        token_max_len=16, token_min_len=1, lower=False)
+        row = wc.get_texts()
+        list_tokens = next(row)
+        self.assertTrue(u'Anarchism' in list_tokens)
+        self.assertTrue(u'collectivization' in list_tokens)
+        self.assertTrue(u'a' in list_tokens)
+        self.assertTrue(u'i.e.' in list_tokens)
+
+    def test_lower_case_set_true(self):
+        """
+        Set the parameter lower to True and check that upper case 'Anarchism' token doesnt exist
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, lower=True, lemmatize=False)
+        row = corpus.get_texts()
+        list_tokens = next(row)
+        self.assertTrue(u'Anarchism' not in list_tokens)
+        self.assertTrue(u'anarchism' in list_tokens)
+
+    def test_lower_case_set_false(self):
+        """
+        Set the parameter lower to False and check that upper case Anarchism' token exists
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, lower=False, lemmatize=False)
+        row = corpus.get_texts()
+        list_tokens = next(row)
+        self.assertTrue(u'Anarchism' in list_tokens)
+        self.assertTrue(u'anarchism' in list_tokens)
+
+    def test_min_token_len_not_set(self):
+        """
+        Don't set the parameter token_min_len and check that 'a' as a token doesn't exist
+        Default token_min_len=2
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, lemmatize=False)
+        self.assertTrue(u'a' not in next(corpus.get_texts()))
+
+    def test_min_token_len_set(self):
+        """
+        Set the parameter token_min_len to 1 and check that 'a' as a token exists
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, token_min_len=1, lemmatize=False)
+        self.assertTrue(u'a' in next(corpus.get_texts()))
+
+    def test_max_token_len_not_set(self):
+        """
+        Don't set the parameter token_max_len and check that 'collectivisation' as a token doesn't exist
+        Default token_max_len=15
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, lemmatize=False)
+        self.assertTrue(u'collectivization' not in next(corpus.get_texts()))
+
+    def test_max_token_len_set(self):
+        """
+        Set the parameter token_max_len to 16 and check that 'collectivisation' as a token exists
+        """
+        corpus = self.corpus_class(self.enwiki, processes=1, token_max_len=16, lemmatize=False)
+        self.assertTrue(u'collectivization' in next(corpus.get_texts()))
+
+    # #TODO: sporadic failure to be investigated
+    # def test_get_texts_returns_generator_of_lists(self):
+    #     corpus = self.corpus_class(self.enwiki)
+    #     l = corpus.get_texts()
+    #     self.assertEqual(type(l), types.GeneratorType)
+    #     first = next(l)
+    #     self.assertEqual(type(first), list)
+    #     self.assertTrue(isinstance(first[0], bytes) or isinstance(first[0], str))
+
+    def test_sample_text(self):
+        # Cannot instantiate WikiCorpus from lines
+        pass
+
+    def test_sample_text_length(self):
+        # Cannot instantiate WikiCorpus from lines
+        pass
+
+    def test_sample_text_seed(self):
+        # Cannot instantiate WikiCorpus from lines
+        pass
+
+    def test_empty_input(self):
+        # An empty file is not legit XML
+        pass
+
+
 class TestTextDirectoryCorpus(unittest.TestCase):
 
     def write_one_level(self, *args):
@@ -459,7 +723,7 @@ class TestTextDirectoryCorpus(unittest.TestCase):
         corpus = textcorpus.TextDirectoryCorpus(dirpath, pattern="test.*\.log")
         filenames = list(corpus.iter_filepaths())
         expected = [os.path.join(dirpath, name) for name in ('test1.log', 'test2.log')]
-        self.assertEqual(expected, filenames)
+        self.assertEqual(sorted(expected), sorted(filenames))
 
         corpus.pattern = ".*.txt"
         filenames = list(corpus.iter_filepaths())
@@ -493,7 +757,7 @@ class TestTextDirectoryCorpus(unittest.TestCase):
         .
         ├── 0.txt
         ├── a_folder
-        │   └── 1.txt
+        │   └── 1.txt
         └── b_folder
             ├── 2.txt
             ├── 3.txt
