@@ -12,18 +12,19 @@ Automated tests for similarity algorithms (the similarities package).
 import logging
 import unittest
 import os
-import tempfile
 
 import numpy
 import scipy
 
+from smart_open import smart_open
 from gensim.corpora import Dictionary
 from gensim.models import word2vec
 from gensim.models import doc2vec
 from gensim.models import KeyedVectors
-from gensim.models.wrappers import fasttext
 from gensim import matutils, similarities
-from gensim.models import Word2Vec
+from gensim.models import Word2Vec, FastText
+from gensim.test.utils import (datapath, get_tmpfile,
+    common_texts as texts, common_dictionary as dictionary, common_corpus as corpus)
 
 try:
     from pyemd import emd  # noqa:F401
@@ -31,38 +32,18 @@ try:
 except ImportError:
     PYEMD_EXT = False
 
-module_path = os.path.dirname(__file__)  # needed because sample data files are located in the same folder
-datapath = lambda fname: os.path.join(module_path, 'test_data', fname)
-
-
-# set up vars used in testing ("Deerwester" from the web tutorial)
-texts = [
-    ['human', 'interface', 'computer'],
-    ['survey', 'user', 'computer', 'system', 'response', 'time'],
-    ['eps', 'user', 'interface', 'system'],
-    ['system', 'human', 'system', 'eps'],
-    ['user', 'response', 'time'],
-    ['trees'],
-    ['graph', 'trees'],
-    ['graph', 'minors', 'trees'],
-    ['graph', 'minors', 'survey']
-]
-dictionary = Dictionary(texts)
-corpus = [dictionary.doc2bow(text) for text in texts]
-
 sentences = [doc2vec.TaggedDocument(words, [i]) for i, words in enumerate(texts)]
 
 
-def testfile():
-    # temporary data will be stored to this file
-    return os.path.join(tempfile.gettempdir(), 'gensim_similarities.tst.pkl')
-
-
 class _TestSimilarityABC(object):
     """
     Base class for SparseMatrixSimilarity and MatrixSimilarity unit tests.
     """
 
+    def factoryMethod(self):
+        """Creates a SimilarityABC instance."""
+        return self.cls(corpus, num_features=len(dictionary))
+
     def testFull(self, num_best=None, shardsize=100):
         if self.cls == similarities.Similarity:
             index = self.cls(None, corpus, num_features=len(dictionary), shardsize=shardsize)
@@ -127,6 +108,15 @@ class _TestSimilarityABC(object):
         self.assertTrue(scipy.sparse.issparse(matrix_scipy_clipped))
         self.assertTrue([matutils.scipy2sparse(x) for x in matrix_scipy_clipped], [expected] * 3)
 
+    def testEmptyQuery(self):
+        index = self.factoryMethod()
+        query = []
+        try:
+            sims = index[query]
+            self.assertTrue(sims is not None)
+        except IndexError:
+            self.assertTrue(False)
+
     def testChunking(self):
         if self.cls == similarities.Similarity:
             index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
@@ -178,13 +168,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile()
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
+        index = self.factoryMethod()
         index.save(fname)
         index2 = self.cls.load(fname)
         if self.cls == similarities.Similarity:
@@ -203,13 +188,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile() + '.gz'
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl.gz')
+        index = self.factoryMethod()
         index.save(fname)
         index2 = self.cls.load(fname)
         if self.cls == similarities.Similarity:
@@ -228,13 +208,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile()
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
+        index = self.factoryMethod()
         # store all arrays separately
         index.save(fname, sep_limit=0)
 
@@ -255,13 +230,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile() + '.gz'
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl.gz')
+        index = self.factoryMethod()
         # store all arrays separately
         index.save(fname, sep_limit=0)
 
@@ -282,13 +252,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile()
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
+        index = self.factoryMethod()
         # store all arrays separately
         index.save(fname, sep_limit=0)
 
@@ -310,13 +275,8 @@ class _TestSimilarityABC(object):
         if self.cls == similarities.WmdSimilarity and not PYEMD_EXT:
             return
 
-        fname = testfile() + '.gz'
-        if self.cls == similarities.Similarity:
-            index = self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
-        elif self.cls == similarities.WmdSimilarity:
-            index = self.cls(texts, self.w2v_model)
-        else:
-            index = self.cls(corpus, num_features=len(dictionary))
+        fname = get_tmpfile('gensim_similarities.tst.pkl.gz')
+        index = self.factoryMethod()
         # store all arrays separately
         index.save(fname, sep_limit=0)
 
@@ -334,6 +294,10 @@ class TestWmdSimilarity(unittest.TestCase, _TestSimilarityABC):
         self.cls = similarities.WmdSimilarity
         self.w2v_model = Word2Vec(texts, min_count=1)
 
+    def factoryMethod(self):
+        # Override factoryMethod.
+        return self.cls(texts, self.w2v_model)
+
     def testFull(self, num_best=None):
         # Override testFull.
 
@@ -348,7 +312,8 @@ class TestWmdSimilarity(unittest.TestCase, _TestSimilarityABC):
         if num_best is not None:
             # Sparse array.
             for i, sim in sims:
-                self.assertTrue(numpy.alltrue(sim > 0.0))  # Note that similarities are bigger than zero, as they are the 1/ 1 + distances.
+                # Note that similarities are bigger than zero, as they are the 1/ 1 + distances.
+                self.assertTrue(numpy.alltrue(sim > 0.0))
         else:
             self.assertTrue(sims[0] == 1.0)  # Similarity of a document with itself is 0.0.
             self.assertTrue(numpy.alltrue(sims[1:] > 0.0))
@@ -405,6 +370,80 @@ class TestWmdSimilarity(unittest.TestCase, _TestSimilarityABC):
             self.assertTrue(numpy.alltrue(sims <= 1.0))
 
 
+class TestSoftCosineSimilarity(unittest.TestCase, _TestSimilarityABC):
+    def setUp(self):
+        self.cls = similarities.SoftCosineSimilarity
+        self.dictionary = Dictionary(texts)
+        self.corpus = [dictionary.doc2bow(document) for document in texts]
+        similarity_matrix = scipy.sparse.identity(12, format="lil")
+        similarity_matrix[dictionary.token2id["user"], dictionary.token2id["human"]] = 0.5
+        similarity_matrix[dictionary.token2id["human"], dictionary.token2id["user"]] = 0.5
+        self.similarity_matrix = similarity_matrix.tocsc()
+
+    def factoryMethod(self):
+        # Override factoryMethod.
+        return self.cls(self.corpus, self.similarity_matrix)
+
+    def testFull(self, num_best=None):
+        # Override testFull.
+
+        index = self.cls(self.corpus, self.similarity_matrix, num_best=num_best)
+        query = self.dictionary.doc2bow(texts[0])
+        sims = index[query]
+
+        if num_best is not None:
+            # Sparse array.
+            for i, sim in sims:
+                self.assertTrue(numpy.alltrue(sim <= 1.0))
+                self.assertTrue(numpy.alltrue(sim >= 0.0))
+        else:
+            self.assertTrue(sims[0] == 1.0)  # Similarity of a document with itself is 1.0.
+            self.assertTrue(numpy.alltrue(sims[1:] >= 0.0))
+            self.assertTrue(numpy.alltrue(sims[1:] < 1.0))
+            expected = 2.1889350195476758
+            self.assertAlmostEqual(expected, numpy.sum(sims))
+
+    def testNonIncreasing(self):
+        """ Check that similarities are non-increasing when `num_best` is not `None`."""
+        # NOTE: this could be implemented for other similarities as well (i.e. in _TestSimilarityABC).
+
+        index = self.cls(self.corpus, self.similarity_matrix, num_best=5)
+        query = self.dictionary.doc2bow(texts[0])
+        sims = index[query]
+        sims2 = numpy.asarray(sims)[:, 1]  # Just the similarities themselves.
+
+        # The difference of adjacent elements should be negative.
+        cond = sum(numpy.diff(sims2) < 0) == len(sims2) - 1
+        self.assertTrue(cond)
+
+    def testChunking(self):
+        # Override testChunking.
+
+        index = self.cls(self.corpus, self.similarity_matrix)
+        query = [self.dictionary.doc2bow(document) for document in texts[:3]]
+        sims = index[query]
+
+        for i in range(3):
+            self.assertTrue(numpy.alltrue(sims[i, i] == 1.0))  # Similarity of a document with itself is 1.0.
+
+        # test the same thing but with num_best
+        index.num_best = 5
+        sims = index[query]
+        for i, chunk in enumerate(sims):
+            expected = i
+            self.assertAlmostEquals(expected, chunk[0][0], places=2)
+            expected = 1.0
+            self.assertAlmostEquals(expected, chunk[0][1], places=2)
+
+    def testIter(self):
+        # Override testIter.
+
+        index = self.cls(self.corpus, self.similarity_matrix)
+        for sims in index:
+            self.assertTrue(numpy.alltrue(sims >= 0.0))
+            self.assertTrue(numpy.alltrue(sims <= 1.0))
+
+
 class TestSparseMatrixSimilarity(unittest.TestCase, _TestSimilarityABC):
     def setUp(self):
         self.cls = similarities.SparseMatrixSimilarity
@@ -442,6 +481,10 @@ class TestSimilarity(unittest.TestCase, _TestSimilarityABC):
     def setUp(self):
         self.cls = similarities.Similarity
 
+    def factoryMethod(self):
+        # Override factoryMethod.
+        return self.cls(None, corpus, num_features=len(dictionary), shardsize=5)
+
     def testSharding(self):
         for num_best in [None, 0, 1, 9, 1000]:
             for shardsize in [1, 2, 9, 1000]:
@@ -495,12 +538,16 @@ class TestWord2VecAnnoyIndexer(unittest.TestCase):
         self.assertLoadedIndexEqual(index, model)
 
     def testFastText(self):
-        ft_home = os.environ.get('FT_HOME', None)
-        ft_path = os.path.join(ft_home, 'fasttext') if ft_home else None
-        if not ft_path:
-            return
-        corpus_file = datapath('lee.cor')
-        model = fasttext.FastText.train(ft_path, corpus_file)
+        class LeeReader(object):
+            def __init__(self, fn):
+                self.fn = fn
+
+            def __iter__(self):
+                with smart_open(self.fn, 'r', encoding="latin_1") as infile:
+                    for line in infile:
+                        yield line.lower().strip().split()
+
+        model = FastText(LeeReader(datapath('lee.cor')))
         model.init_sims()
         index = self.indexer(model, 10)
 
@@ -532,7 +579,7 @@ class TestWord2VecAnnoyIndexer(unittest.TestCase):
         word, similarity = approx_neighbors[0]
 
         self.assertEqual(word, label)
-        self.assertEqual(similarity, 1.0)
+        self.assertAlmostEqual(similarity, 1.0, places=2)
 
     def assertApproxNeighborsMatchExact(self, model, wv, index):
         vector = wv.syn0norm[0]
@@ -545,7 +592,7 @@ class TestWord2VecAnnoyIndexer(unittest.TestCase):
         self.assertEqual(approx_words, exact_words)
 
     def assertIndexSaved(self, index):
-        fname = testfile()
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
         index.save(fname)
         self.assertTrue(os.path.exists(fname))
         self.assertTrue(os.path.exists(fname + '.d'))
@@ -553,7 +600,7 @@ class TestWord2VecAnnoyIndexer(unittest.TestCase):
     def assertLoadedIndexEqual(self, index, model):
         from gensim.similarities.index import AnnoyIndexer
 
-        fname = testfile()
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
         index.save(fname)
 
         index2 = AnnoyIndexer()
@@ -585,7 +632,7 @@ class TestDoc2VecAnnoyIndexer(unittest.TestCase):
         doc, similarity = approx_neighbors[0]
 
         self.assertEqual(doc, 0)
-        self.assertEqual(similarity, 1.0)
+        self.assertAlmostEqual(similarity, 1.0, places=2)
 
     def testApproxNeighborsMatchExact(self):
         approx_neighbors = self.model.docvecs.most_similar([self.vector], topn=5, indexer=self.index)
@@ -598,7 +645,7 @@ class TestDoc2VecAnnoyIndexer(unittest.TestCase):
         self.assertEqual(approx_words, exact_words)
 
     def testSave(self):
-        fname = testfile()
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
         self.index.save(fname)
         self.assertTrue(os.path.exists(fname))
         self.assertTrue(os.path.exists(fname + '.d'))
@@ -612,7 +659,7 @@ class TestDoc2VecAnnoyIndexer(unittest.TestCase):
     def testSaveLoad(self):
         from gensim.similarities.index import AnnoyIndexer
 
-        fname = testfile()
+        fname = get_tmpfile('gensim_similarities.tst.pkl')
         self.index.save(fname)
 
         self.index2 = AnnoyIndexer()
