@@ -12,7 +12,6 @@ Automated tests for checking transformation algorithms (the models package).
 import logging
 import unittest
 import os
-import tempfile
 import bz2
 import sys
 
@@ -20,6 +19,7 @@ import numpy as np
 
 from gensim import utils
 from gensim.models import word2vec, keyedvectors
+from gensim.test.utils import datapath, get_tmpfile, common_texts as sentences
 from testfixtures import log_capture
 
 try:
@@ -28,9 +28,6 @@ try:
 except ImportError:
     PYEMD_EXT = False
 
-module_path = os.path.dirname(__file__)  # needed because sample data files are located in the same folder
-datapath = lambda fname: os.path.join(module_path, 'test_data', fname)
-
 
 class LeeCorpus(object):
     def __iter__(self):
@@ -41,18 +38,6 @@ class LeeCorpus(object):
 
 list_corpus = list(LeeCorpus())
 
-sentences = [
-    ['human', 'interface', 'computer'],
-    ['survey', 'user', 'computer', 'system', 'response', 'time'],
-    ['eps', 'user', 'interface', 'system'],
-    ['system', 'human', 'system', 'eps'],
-    ['user', 'response', 'time'],
-    ['trees'],
-    ['graph', 'trees'],
-    ['graph', 'minors', 'trees'],
-    ['graph', 'minors', 'survey']
-]
-
 new_sentences = [
     ['computer', 'artificial', 'intelligence'],
     ['artificial', 'trees'],
@@ -63,11 +48,6 @@ new_sentences = [
 ]
 
 
-def testfile():
-    # temporary data will be stored to this file
-    return os.path.join(tempfile.gettempdir(), 'gensim_word2vec.tst')
-
-
 def _rule(word, count, min_count):
     if word == "human":
         return utils.RULE_DISCARD  # throw out
@@ -77,13 +57,94 @@ def _rule(word, count, min_count):
 
 def load_on_instance():
     # Save and load a Word2Vec Model on instance for test
+    tmpf = get_tmpfile('gensim_word2vec.tst')
     model = word2vec.Word2Vec(sentences, min_count=1)
-    model.save(testfile())
+    model.save(tmpf)
     model = word2vec.Word2Vec()  # should fail at this point
-    return model.load(testfile())
+    return model.load(tmpf)
 
 
 class TestWord2VecModel(unittest.TestCase):
+    def testBuildVocabFromFreq(self):
+        """Test that the algorithm is able to build vocabulary from given
+        frequency table"""
+        freq_dict = {
+        'minors': 2, 'graph': 3, 'system': 4,
+        'trees': 3, 'eps': 2, 'computer': 2,
+        'survey': 2, 'user': 3, 'human': 2,
+        'time': 2, 'interface': 2, 'response': 2
+        }
+        model_hs = word2vec.Word2Vec(size=10, min_count=0, seed=42, hs=1, negative=0)
+        model_neg = word2vec.Word2Vec(size=10, min_count=0, seed=42, hs=0, negative=5)
+        model_hs.build_vocab_from_freq(freq_dict)
+        model_neg.build_vocab_from_freq(freq_dict)
+        self.assertEqual(len(model_hs.wv.vocab), 12)
+        self.assertEqual(len(model_neg.wv.vocab), 12)
+        self.assertEqual(model_hs.wv.vocab['minors'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['graph'].count, 3)
+        self.assertEqual(model_hs.wv.vocab['system'].count, 4)
+        self.assertEqual(model_hs.wv.vocab['trees'].count, 3)
+        self.assertEqual(model_hs.wv.vocab['eps'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['computer'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['survey'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['user'].count, 3)
+        self.assertEqual(model_hs.wv.vocab['human'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['time'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['interface'].count, 2)
+        self.assertEqual(model_hs.wv.vocab['response'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['minors'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['graph'].count, 3)
+        self.assertEqual(model_neg.wv.vocab['system'].count, 4)
+        self.assertEqual(model_neg.wv.vocab['trees'].count, 3)
+        self.assertEqual(model_neg.wv.vocab['eps'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['computer'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['survey'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['user'].count, 3)
+        self.assertEqual(model_neg.wv.vocab['human'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['time'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['interface'].count, 2)
+        self.assertEqual(model_neg.wv.vocab['response'].count, 2)
+        new_freq_dict = {
+            'computer': 1, 'artificial': 4, 'human': 1, 'graph': 1, 'intelligence': 4, 'system': 1, 'trees': 1
+        }
+        model_hs.build_vocab_from_freq(new_freq_dict, update=True)
+        model_neg.build_vocab_from_freq(new_freq_dict, update=True)
+        self.assertEqual(model_hs.wv.vocab['graph'].count, 4)
+        self.assertEqual(model_hs.wv.vocab['artificial'].count, 4)
+        self.assertEqual(len(model_hs.wv.vocab), 14)
+        self.assertEqual(len(model_neg.wv.vocab), 14)
+
+    def testPruneVocab(self):
+        """Test Prune vocab while scanning sentences"""
+        sentences = [
+            ["graph", "system"],
+            ["graph", "system"],
+            ["system", "eps"],
+            ["graph", "system"]
+        ]
+        model = word2vec.Word2Vec(sentences, size=10, min_count=0, max_vocab_size=2, seed=42, hs=1, negative=0)
+        self.assertEqual(len(model.wv.vocab), 2)
+        self.assertEqual(model.wv.vocab['graph'].count, 3)
+        self.assertEqual(model.wv.vocab['system'].count, 4)
+
+        sentences = [
+            ["graph", "system"],
+            ["graph", "system"],
+            ["system", "eps"],
+            ["graph", "system"],
+            ["minors", "survey", "minors", "survey", "minors"]
+        ]
+        model = word2vec.Word2Vec(sentences, size=10, min_count=0, max_vocab_size=2, seed=42, hs=1, negative=0)
+        self.assertEqual(len(model.wv.vocab), 3)
+        self.assertEqual(model.wv.vocab['graph'].count, 3)
+        self.assertEqual(model.wv.vocab['minors'].count, 3)
+        self.assertEqual(model.wv.vocab['system'].count, 4)
+
+    def testTotalWordCount(self):
+        model = word2vec.Word2Vec(size=10, min_count=0, seed=42)
+        total_words = model.vocabulary.scan_vocab(sentences)[0]
+        self.assertEqual(total_words, 29)
+
     def testOnlineLearning(self):
         """Test that the algorithm is able to add new words to the
         vocabulary and to a trained model when using a sorted vocabulary"""
@@ -101,9 +162,10 @@ class TestWord2VecModel(unittest.TestCase):
     def testOnlineLearningAfterSave(self):
         """Test that the algorithm is able to add new words to the
         vocabulary and to a trained model when using a sorted vocabulary"""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model_neg = word2vec.Word2Vec(sentences, size=10, min_count=0, seed=42, hs=0, negative=5)
-        model_neg.save(testfile())
-        model_neg = word2vec.Word2Vec.load(testfile())
+        model_neg.save(tmpf)
+        model_neg = word2vec.Word2Vec.load(tmpf)
         self.assertTrue(len(model_neg.wv.vocab), 12)
         model_neg.build_vocab(new_sentences, update=True)
         model_neg.train(new_sentences, total_examples=model_neg.corpus_count, epochs=model_neg.iter)
@@ -156,21 +218,23 @@ class TestWord2VecModel(unittest.TestCase):
 
     def testPersistence(self):
         """Test storing/loading the entire model."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
-        model.save(testfile())
-        self.models_equal(model, word2vec.Word2Vec.load(testfile()))
+        model.save(tmpf)
+        self.models_equal(model, word2vec.Word2Vec.load(tmpf))
         #  test persistence of the KeyedVectors of a model
         wv = model.wv
-        wv.save(testfile())
-        loaded_wv = keyedvectors.KeyedVectors.load(testfile())
+        wv.save(tmpf)
+        loaded_wv = keyedvectors.KeyedVectors.load(tmpf)
         self.assertTrue(np.allclose(wv.syn0, loaded_wv.syn0))
         self.assertEqual(len(wv.vocab), len(loaded_wv.vocab))
 
     def testPersistenceWithConstructorRule(self):
         """Test storing/loading the entire model with a vocab trimming rule passed in the constructor."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1, trim_rule=_rule)
-        model.save(testfile())
-        self.models_equal(model, word2vec.Word2Vec.load(testfile()))
+        model.save(tmpf)
+        self.models_equal(model, word2vec.Word2Vec.load(tmpf))
 
     def testRuleWithMinCount(self):
         """Test that returning RULE_DEFAULT from trim_rule triggers min_count."""
@@ -187,21 +251,24 @@ class TestWord2VecModel(unittest.TestCase):
 
     def testLambdaRule(self):
         """Test that lambda trim_rule works."""
-        rule = lambda word, count, min_count: utils.RULE_DISCARD if word == "human" else utils.RULE_DEFAULT
+        def rule(word, count, min_count):
+            return utils.RULE_DISCARD if word == "human" else utils.RULE_DEFAULT
+
         model = word2vec.Word2Vec(sentences, min_count=1, trim_rule=rule)
         self.assertTrue("human" not in model.wv.vocab)
 
     def testSyn0NormNotSaved(self):
         """Test syn0norm isn't saved in model file"""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        model.save(testfile())
-        loaded_model = word2vec.Word2Vec.load(testfile())
+        model.save(tmpf)
+        loaded_model = word2vec.Word2Vec.load(tmpf)
         self.assertTrue(loaded_model.wv.syn0norm is None)
 
         wv = model.wv
-        wv.save(testfile())
-        loaded_kv = keyedvectors.KeyedVectors.load(testfile())
+        wv.save(tmpf)
+        loaded_kv = keyedvectors.KeyedVectors.load(tmpf)
         self.assertTrue(loaded_kv.syn0norm is None)
 
     def testLoadPreKeyedVectorModel(self):
@@ -233,34 +300,36 @@ class TestWord2VecModel(unittest.TestCase):
 
     def testPersistenceWord2VecFormat(self):
         """Test storing/loading the entire model in word2vec format."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        model.wv.save_word2vec_format(testfile(), binary=True)
-        binary_model_kv = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=True)
+        model.wv.save_word2vec_format(tmpf, binary=True)
+        binary_model_kv = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=True)
         binary_model_kv.init_sims(replace=False)
         self.assertTrue(np.allclose(model['human'], binary_model_kv['human']))
-        norm_only_model = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=True)
+        norm_only_model = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=True)
         norm_only_model.init_sims(replace=True)
         self.assertFalse(np.allclose(model['human'], norm_only_model['human']))
         self.assertTrue(np.allclose(model.wv.syn0norm[model.wv.vocab['human'].index], norm_only_model['human']))
-        limited_model_kv = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=True, limit=3)
-        self.assertEquals(len(limited_model_kv.syn0), 3)
+        limited_model_kv = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=True, limit=3)
+        self.assertEqual(len(limited_model_kv.syn0), 3)
         half_precision_model_kv = keyedvectors.KeyedVectors.load_word2vec_format(
-            testfile(), binary=True, datatype=np.float16
+            tmpf, binary=True, datatype=np.float16
         )
-        self.assertEquals(binary_model_kv.syn0.nbytes, half_precision_model_kv.syn0.nbytes * 2)
+        self.assertEqual(binary_model_kv.syn0.nbytes, half_precision_model_kv.syn0.nbytes * 2)
 
     def testNoTrainingCFormat(self):
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        model.wv.save_word2vec_format(testfile(), binary=True)
-        kv = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=True)
+        model.wv.save_word2vec_format(tmpf, binary=True)
+        kv = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=True)
         binary_model = word2vec.Word2Vec()
         binary_model.wv = kv
         self.assertRaises(ValueError, binary_model.train, sentences)
 
     def testTooShortBinaryWord2VecFormat(self):
-        tfile = testfile()
+        tfile = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
         model.wv.save_word2vec_format(tfile, binary=True)
@@ -270,7 +339,7 @@ class TestWord2VecModel(unittest.TestCase):
         self.assertRaises(EOFError, keyedvectors.KeyedVectors.load_word2vec_format, tfile, binary=True)
 
     def testTooShortTextWord2VecFormat(self):
-        tfile = testfile()
+        tfile = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
         model.wv.save_word2vec_format(tfile, binary=False)
@@ -281,13 +350,14 @@ class TestWord2VecModel(unittest.TestCase):
 
     def testPersistenceWord2VecFormatNonBinary(self):
         """Test storing/loading the entire model in word2vec non-binary format."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        model.wv.save_word2vec_format(testfile(), binary=False)
-        text_model = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=False)
+        model.wv.save_word2vec_format(tmpf, binary=False)
+        text_model = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=False)
         text_model.init_sims(False)
         self.assertTrue(np.allclose(model['human'], text_model['human'], atol=1e-6))
-        norm_only_model = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=False)
+        norm_only_model = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=False)
         norm_only_model.init_sims(True)
         self.assertFalse(np.allclose(model['human'], norm_only_model['human'], atol=1e-6))
         self.assertTrue(np.allclose(
@@ -296,44 +366,48 @@ class TestWord2VecModel(unittest.TestCase):
 
     def testPersistenceWord2VecFormatWithVocab(self):
         """Test storing/loading the entire model and vocabulary in word2vec format."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        testvocab = os.path.join(tempfile.gettempdir(), 'gensim_word2vec.vocab')
-        model.wv.save_word2vec_format(testfile(), testvocab, binary=True)
-        binary_model_with_vocab_kv = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), testvocab, binary=True)
+        testvocab = get_tmpfile('gensim_word2vec.vocab')
+        model.wv.save_word2vec_format(tmpf, testvocab, binary=True)
+        binary_model_with_vocab_kv = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, testvocab, binary=True)
         self.assertEqual(model.wv.vocab['human'].count, binary_model_with_vocab_kv.vocab['human'].count)
 
     def testPersistenceKeyedVectorsFormatWithVocab(self):
         """Test storing/loading the entire model and vocabulary in word2vec format."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        testvocab = os.path.join(tempfile.gettempdir(), 'gensim_word2vec.vocab')
-        model.wv.save_word2vec_format(testfile(), testvocab, binary=True)
-        kv_binary_model_with_vocab = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), testvocab, binary=True)
+        testvocab = get_tmpfile('gensim_word2vec.vocab')
+        model.wv.save_word2vec_format(tmpf, testvocab, binary=True)
+        kv_binary_model_with_vocab = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, testvocab, binary=True)
         self.assertEqual(model.wv.vocab['human'].count, kv_binary_model_with_vocab.vocab['human'].count)
 
     def testPersistenceWord2VecFormatCombinationWithStandardPersistence(self):
         """Test storing/loading the entire model and vocabulary in word2vec format chained with
          saving and loading via `save` and `load` methods`.
          It was possible prior to 1.0.0 release, now raises Exception"""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
         model.init_sims()
-        testvocab = os.path.join(tempfile.gettempdir(), 'gensim_word2vec.vocab')
-        model.wv.save_word2vec_format(testfile(), testvocab, binary=True)
-        binary_model_with_vocab_kv = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), testvocab, binary=True)
-        binary_model_with_vocab_kv.save(testfile())
-        self.assertRaises(AttributeError, word2vec.Word2Vec.load, testfile())
+        testvocab = get_tmpfile('gensim_word2vec.vocab')
+        model.wv.save_word2vec_format(tmpf, testvocab, binary=True)
+        binary_model_with_vocab_kv = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, testvocab, binary=True)
+        binary_model_with_vocab_kv.save(tmpf)
+        self.assertRaises(AttributeError, word2vec.Word2Vec.load, tmpf)
 
     def testLargeMmap(self):
         """Test storing/loading the entire model."""
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
 
         # test storing the internal arrays into separate files
-        model.save(testfile(), sep_limit=0)
-        self.models_equal(model, word2vec.Word2Vec.load(testfile()))
+        model.save(tmpf, sep_limit=0)
+        self.models_equal(model, word2vec.Word2Vec.load(tmpf))
 
         # make sure mmaping the arrays back works, too
-        self.models_equal(model, word2vec.Word2Vec.load(testfile(), mmap='r'))
+        self.models_equal(model, word2vec.Word2Vec.load(tmpf, mmap='r'))
 
     def testVocab(self):
         """Test word2vec vocabulary building."""
@@ -344,7 +418,8 @@ class TestWord2VecModel(unittest.TestCase):
         model = word2vec.Word2Vec(min_count=1, hs=1, negative=0)
         model.build_vocab(corpus)
         self.assertTrue(len(model.wv.vocab) == 6981)
-        # with min_count=1, we're not throwing away anything, so make sure the word counts add up to be the entire corpus
+        # with min_count=1, we're not throwing away anything,
+        # so make sure the word counts add up to be the entire corpus
         self.assertEqual(sum(v.count for v in model.wv.vocab.values()), total_words)
         # make sure the binary codes are correct
         np.allclose(model.wv.vocab['the'].code, [1, 1, 0, 0])
@@ -625,9 +700,10 @@ class TestWord2VecModel(unittest.TestCase):
                 self.assertTrue(not hasattr(model, 'syn0_lockf'))
 
     def testNormalizeAfterTrainingData(self):
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model = word2vec.Word2Vec(sentences, min_count=1)
-        model.save(testfile())
-        norm_only_model = word2vec.Word2Vec.load(testfile())
+        model.save(tmpf)
+        norm_only_model = word2vec.Word2Vec.load(tmpf)
         norm_only_model.delete_temporary_training_data(replace_word_vectors_with_normalized=True)
         self.assertFalse(np.allclose(model['human'], norm_only_model['human']))
 
@@ -643,9 +719,10 @@ class TestWord2VecModel(unittest.TestCase):
         self.assertEqual(predictions_out_of_vocab, None)
 
         # when required model parameters have been deleted
+        tmpf = get_tmpfile('gensim_word2vec.tst')
         model_with_neg.init_sims()
-        model_with_neg.wv.save_word2vec_format(testfile(), binary=True)
-        kv_model_with_neg = keyedvectors.KeyedVectors.load_word2vec_format(testfile(), binary=True)
+        model_with_neg.wv.save_word2vec_format(tmpf, binary=True)
+        kv_model_with_neg = keyedvectors.KeyedVectors.load_word2vec_format(tmpf, binary=True)
         binary_model_with_neg = word2vec.Word2Vec()
         binary_model_with_neg.wv = kv_model_with_neg
         self.assertRaises(RuntimeError, binary_model_with_neg.predict_output_word, ['system', 'human'])
@@ -654,6 +731,28 @@ class TestWord2VecModel(unittest.TestCase):
         model_without_neg = word2vec.Word2Vec(sentences, min_count=1, negative=0)
         self.assertRaises(RuntimeError, model_without_neg.predict_output_word, ['system', 'human'])
 
+    def testLoadOldModel(self):
+        """Test loading word2vec models from previous version"""
+
+        model_file = 'word2vec_old'
+        model = word2vec.Word2Vec.load(datapath(model_file))
+        self.assertTrue(model.wv.vectors.shape == (12, 100))
+        self.assertTrue(len(model.wv.vocab) == 12)
+        self.assertTrue(len(model.wv.index2word) == 12)
+        self.assertTrue(model.syn1neg.shape == (len(model.wv.vocab), model.wv.vector_size))
+        self.assertTrue(model.trainables.vectors_lockf.shape == (12,))
+        self.assertTrue(model.vocabulary.cum_table.shape == (12,))
+
+        # Model stored in multiple files
+        model_file = 'word2vec_old_sep'
+        model = word2vec.Word2Vec.load(datapath(model_file))
+        self.assertTrue(model.wv.vectors.shape == (12, 100))
+        self.assertTrue(len(model.wv.vocab) == 12)
+        self.assertTrue(len(model.wv.index2word) == 12)
+        self.assertTrue(model.syn1neg.shape == (len(model.wv.vocab), model.wv.vector_size))
+        self.assertTrue(model.trainables.vectors_lockf.shape == (12,))
+        self.assertTrue(model.vocabulary.cum_table.shape == (12,))
+
     @log_capture()
     def testBuildVocabWarning(self, l):
         """Test if warning is raised on non-ideal input to a word2vec model"""
@@ -811,7 +910,8 @@ class TestWord2VecSentenceIterators(unittest.TestCase):
 # class TestWord2VecScripts(unittest.TestCase):
 #     def testWord2VecStandAloneScript(self):
 #         """Does Word2Vec script launch standalone?"""
-#         cmd = 'python -m gensim.scripts.word2vec_standalone -train ' + datapath('testcorpus.txt') + ' -output vec.txt -size 200 -sample 1e-4 -binary 0 -iter 3 -min_count 1'
+#         cmd = 'python -m gensim.scripts.word2vec_standalone -train ' + datapath('testcorpus.txt') + \
+#               ' -output vec.txt -size 200 -sample 1e-4 -binary 0 -iter 3 -min_count 1'
 #         output = check_output(cmd, stderr=PIPE)
 #         self.assertEqual(output, '0')
 # #endclass TestWord2VecScripts
